import os
import unittest

from conans.client.tools.files import rename, chdir
from conans.test.utils.tools import TestClient


class RenameTest(unittest.TestCase):
    def verify_dir(self, basedir):
        self.assertTrue(os.path.isdir(basedir))

        self.assertTrue(os.path.isfile(os.path.join(basedir, "1.txt")))
        self.assertTrue(os.path.isfile(os.path.join(basedir, "1.pdb")))
        self.assertTrue(os.path.isfile(os.path.join(basedir, "1.pdb1")))

        self.assertTrue(os.path.isdir(os.path.join(basedir, "dir.pdb")))
        self.assertTrue(os.path.isfile(os.path.join(basedir, "dir.pdb", "2.txt")))
        self.assertTrue(os.path.isfile(os.path.join(basedir, "dir.pdb", "2.pdb")))
        self.assertTrue(os.path.isfile(os.path.join(basedir, "dir.pdb", "2.pdb1")))

        self.assertTrue(os.path.isdir(os.path.join(basedir, "middir")))
        self.assertTrue(os.path.isfile(os.path.join(basedir, "middir", "3.txt")))
        self.assertTrue(os.path.isfile(os.path.join(basedir, "middir", "3.pdb")))
        self.assertTrue(os.path.isfile(os.path.join(basedir, "middir", "3.pdb1")))

        self.assertTrue(os.path.isdir(os.path.join(basedir, "middir", "deepdir")))
        self.assertTrue(os.path.isfile(os.path.join(basedir, "middir", "deepdir", "4.txt")))
        self.assertTrue(os.path.isfile(os.path.join(basedir, "middir", "deepdir", "4.pdb")))
        self.assertTrue(os.path.isfile(os.path.join(basedir, "middir", "deepdir", "4.pdb1")))

    def test_rename(self):
        client = TestClient()
        tmpdir = client.current_folder

        sub_space_dir = "sub dir"
        with chdir(tmpdir):
            os.makedirs(sub_space_dir)
            os.makedirs(os.path.join(sub_space_dir, "dir.pdb"))
            os.makedirs(os.path.join(sub_space_dir, "middir"))
            os.makedirs(os.path.join(sub_space_dir, "middir", "deepdir"))

        client.save({os.path.join(sub_space_dir, "1.txt"): "",
                     os.path.join(sub_space_dir, "1.pdb"): "",
                     os.path.join(sub_space_dir, "1.pdb1"): "",
                     os.path.join(sub_space_dir, "dir.pdb", "2.txt"): "",
                     os.path.join(sub_space_dir, "dir.pdb", "2.pdb"): "",
                     os.path.join(sub_space_dir, "dir.pdb", "2.pdb1"): "",
                     os.path.join(sub_space_dir, "middir", "3.txt"): "",
                     os.path.join(sub_space_dir, "middir", "3.pdb"): "",
                     os.path.join(sub_space_dir, "middir", "3.pdb1"): "",
                     os.path.join(sub_space_dir, "middir", "deepdir", "4.txt"): "",
                     os.path.join(sub_space_dir, "middir", "deepdir", "4.pdb"): "",
                     os.path.join(sub_space_dir, "middir", "deepdir", "4.pdb1"): ""
                    })
        self.verify_dir(os.path.join(tmpdir, sub_space_dir))

        with chdir(tmpdir):
            rename(sub_space_dir, "dst dir")
            self.verify_dir(os.path.join(tmpdir, "dst dir"))

            rename("dst dir", "subdir")
            self.verify_dir(os.path.join(tmpdir, "subdir"))

            rename(os.path.join("subdir", "1.txt"), "t.txt")
            self.assertTrue(os.path.isfile(os.path.join(tmpdir, "t.txt")))
            self.assertFalse(os.path.isfile(os.path.join(tmpdir, "subdir", "1.txt")))

    def test_rename_empty_folder(self):
        client = TestClient()
        old_folder = os.path.join(client.current_folder, "old_folder")
        os.mkdir(old_folder)
        new_folder = os.path.join(client.current_folder, "new_folder")
        rename(old_folder, new_folder)
        self.assertFalse(os.path.exists(old_folder))
        self.assertTrue(os.path.exists(new_folder))



import textwrap
import unittest

import pytest

from conans.test.utils.tools import TestClient


class PropagateSpecificComponents(unittest.TestCase):
    """
        Feature: recipes can declare the components they are consuming from their requirements,
        only those components should be propagated to their own consumers. If required components
        doesn't exist, Conan will fail:
         * Resolved versions/revisions of the requirement might provide different components or
           no components at all.
    """

    top = textwrap.dedent("""
        from conans import ConanFile

        class Recipe(ConanFile):
            name = "top"

            def package_info(self):
                self.cpp_info.components["cmp1"].libs = ["top_cmp1"]
                self.cpp_info.components["cmp2"].libs = ["top_cmp2"]
    """)

    middle = textwrap.dedent("""
        from conans import ConanFile

        class Recipe(ConanFile):
            name = "middle"
            requires = "top/version"
            def package_info(self):
                self.cpp_info.requires = ["top::cmp1"]
    """)

    app = textwrap.dedent("""
        from conans import ConanFile

        class Recipe(ConanFile):
            settings = "os", "compiler", "build_type", "arch"
            name = "app"
            requires = "middle/version"
        """)

    def setUp(self):
        client = TestClient()
        client.save({
            'top.py': self.top,
            'middle.py': self.middle,
            'app.py': self.app
        })
        client.run('create top.py top/version@')
        client.run('create middle.py middle/version@')
        self.cache_folder = client.cache_folder

    def test_cmakedeps_app(self):
        t = TestClient(cache_folder=self.cache_folder)
        t.save({'conanfile.py': self.app})
        t.run("install .  -g CMakeDeps")
        config = t.load("middle-Target-release.cmake")
        self.assertIn('top::cmp1', config)
        self.assertNotIn("top::top", config)

    def test_cmakedeps_multi(self):
        t = TestClient(cache_folder=self.cache_folder)
        t.run('install middle/version@ -g CMakeDeps')
        arch = t.get_default_host_profile().settings['arch']
        content = t.load(f'middle-release-{arch}-data.cmake')
        self.assertIn("list(APPEND middle_FIND_DEPENDENCY_NAMES top)", content)

        content = t.load('middle-Target-release.cmake')
        self.assertNotIn("top::top", content)
        self.assertNotIn("top::cmp2", content)
        self.assertIn("top::cmp1", content)


@pytest.fixture
def top_conanfile():
    return textwrap.dedent("""
        from conans import ConanFile

        class Recipe(ConanFile):
            name = "top"

            def package_info(self):
                self.cpp_info.components["cmp1"].libs = ["top_cmp1"]
                self.cpp_info.components["cmp2"].libs = ["top_cmp2"]
    """)


@pytest.mark.parametrize("from_component", [False, True])
def test_wrong_component(top_conanfile, from_component):
    """ If the requirement doesn't provide the component, it fails.
        We can only raise this error after the graph is fully resolved, it is when we
        know the actual components that the requirement is going to provide.
    """

    consumer = textwrap.dedent("""
        from conans import ConanFile

        class Recipe(ConanFile):
            requires = "top/version"
            def package_info(self):
                self.cpp_info.{}requires = ["top::not-existing"]
    """).format("components['foo']." if from_component else "")

    t = TestClient()
    t.save({'top.py': top_conanfile, 'consumer.py': consumer})
    t.run('create top.py top/version@')
    t.run('create consumer.py wrong/version@')

    t.run('install wrong/version@ -g CMakeDeps', assert_error=True)
    assert "Component 'top::not-existing' not found in 'top' package requirement" in t.out


def test_unused_requirement(top_conanfile):
    """ Requires should include all listed requirements
        This error is known when creating the package if the requirement is consumed.
    """
    consumer = textwrap.dedent("""
        from conans import ConanFile

        class Recipe(ConanFile):
            requires = "top/version"
            def package_info(self):
                self.cpp_info.requires = ["other::other"]
    """)
    t = TestClient()
    t.save({'top.py': top_conanfile, 'consumer.py': consumer})
    t.run('create top.py top/version@')
    t.run('create consumer.py wrong/version@', assert_error=True)
    assert "wrong/version package_info(): Package require 'top' not used in components " \
           "requires" in t.out


def test_wrong_requirement(top_conanfile):
    """ If we require a wrong requirement, we get a meaninful error.
        This error is known when creating the package if the requirement is not there.
    """
    consumer = textwrap.dedent("""
        from conans import ConanFile

        class Recipe(ConanFile):
            requires = "top/version"
            def package_info(self):
                self.cpp_info.requires = ["top::cmp1", "other::other"]
    """)
    t = TestClient()
    t.save({'top.py': top_conanfile, 'consumer.py': consumer})
    t.run('create top.py top/version@')
    t.run('create consumer.py wrong/version@', assert_error=True)
    assert "wrong/version package_info(): Package require 'other' declared in " \
           "components requires but not defined as a recipe requirement" in t.out


@pytest.mark.tool_cmake
def test_components_system_libs():
    conanfile = textwrap.dedent("""
        from conans import ConanFile

        class Requirement(ConanFile):
            name = "requirement"
            version = "system"

            settings = "os", "arch", "compiler", "build_type"

            def package_info(self):
                self.cpp_info.components["component"].system_libs = ["system_lib_component"]
    """)
    t = TestClient()
    t.save({"conanfile.py": conanfile})
    t.run("create .")

    conanfile = textwrap.dedent("""
        from conans import ConanFile
        from conan.tools.cmake import CMake

        class Consumer(ConanFile):
            name = "consumer"
            version = "0.1"
            requires = "requirement/system"
            generators = "CMakeDeps", "CMakeToolchain"
            exports_sources = "CMakeLists.txt"
            settings = "os", "arch", "compiler", "build_type"

            def build(self):
                cmake = CMake(self)
                cmake.configure()
    """)

    cmakelists = textwrap.dedent("""
        cmake_minimum_required(VERSION 3.15)
        project(consumer)

        find_package(requirement)
        get_target_property(tmp_libs requirement::component INTERFACE_LINK_LIBRARIES)
        get_target_property(tmp_options requirement::component INTERFACE_LINK_OPTIONS)
        get_target_property(tmp_deps requirement_requirement_component_DEPS_TARGET INTERFACE_LINK_LIBRARIES)
        message("component libs: ${tmp_libs}")
        message("component options: ${tmp_options}")
        message("component deps: ${tmp_deps}")
    """)

    t.save({"conanfile.py": conanfile, "CMakeLists.txt": cmakelists})
    t.run("create . --build missing -s build_type=Release")
    assert 'component libs: $<$<CONFIG:Release>:>;$<$<CONFIG:Release>:>;requirement_requirement_component_DEPS_TARGET' in t.out
    assert 'component deps: $<$<CONFIG:Release>:>;$<$<CONFIG:Release>:system_lib_component>;' in t.out
    assert ('component options: '
            '$<$<CONFIG:Release>:'
            '$<$<STREQUAL:$<TARGET_PROPERTY:TYPE>,SHARED_LIBRARY>:>;'
            '$<$<STREQUAL:$<TARGET_PROPERTY:TYPE>,MODULE_LIBRARY>:>;'
            '$<$<STREQUAL:$<TARGET_PROPERTY:TYPE>,EXECUTABLE>:>>') in t.out
    # NOTE: If there is no "conan install -s build_type=Debug", the properties won't contain the
    #       <CONFIG:Debug>


@pytest.mark.tool_cmake
def test_components_exelinkflags():
    conanfile = textwrap.dedent("""
        from conans import ConanFile

        class Requirement(ConanFile):
            name = "requirement"
            version = "system"

            settings = "os", "arch", "compiler", "build_type"

            def package_info(self):
                self.cpp_info.components["component"].exelinkflags = ["-Wl,-link1", "-Wl,-link2"]
    """)
    t = TestClient()
    t.save({"conanfile.py": conanfile})
    t.run("create .")

    conanfile = textwrap.dedent("""
        from conans import ConanFile
        from conan.tools.cmake import CMake

        class Consumer(ConanFile):
            name = "consumer"
            version = "0.1"
            requires = "requirement/system"
            generators = "CMakeDeps", "CMakeToolchain"
            exports_sources = "CMakeLists.txt"
            settings = "os", "arch", "compiler", "build_type"

            def build(self):
                cmake = CMake(self)
                cmake.configure()
    """)

    cmakelists = textwrap.dedent("""
        cmake_minimum_required(VERSION 3.15)
        project(consumer)
        find_package(requirement)
        get_target_property(tmp_options requirement::component INTERFACE_LINK_OPTIONS)
        message("component options: ${tmp_options}")
    """)

    t.save({"conanfile.py": conanfile, "CMakeLists.txt": cmakelists})
    t.run("create . --build missing -s build_type=Release")
    assert ('component options: '
            '$<$<CONFIG:Release>:'
            '$<$<STREQUAL:$<TARGET_PROPERTY:TYPE>,SHARED_LIBRARY>:>;'
            '$<$<STREQUAL:$<TARGET_PROPERTY:TYPE>,MODULE_LIBRARY>:>;'
            '$<$<STREQUAL:$<TARGET_PROPERTY:TYPE>,EXECUTABLE>:-Wl,-link1;-Wl,-link2>>') in t.out
    # NOTE: If there is no "conan install -s build_type=Debug", the properties won't contain the
    #       <CONFIG:Debug>


@pytest.mark.tool_cmake
def test_components_sharedlinkflags():
    conanfile = textwrap.dedent("""
        from conans import ConanFile

        class Requirement(ConanFile):
            name = "requirement"
            version = "system"

            settings = "os", "arch", "compiler", "build_type"

            def package_info(self):
                self.cpp_info.components["component"].sharedlinkflags = ["-Wl,-link1", "-Wl,-link2"]
    """)
    t = TestClient()
    t.save({"conanfile.py": conanfile})
    t.run("create .")

    conanfile = textwrap.dedent("""
        from conans import ConanFile
        from conan.tools.cmake import CMake

        class Consumer(ConanFile):
            name = "consumer"
            version = "0.1"
            requires = "requirement/system"
            generators = "CMakeDeps", "CMakeToolchain"
            exports_sources = "CMakeLists.txt"
            settings = "os", "arch", "compiler", "build_type"

            def build(self):
                cmake = CMake(self)
                cmake.configure()
    """)

    cmakelists = textwrap.dedent("""
        cmake_minimum_required(VERSION 3.15)
        project(consumer)
        find_package(requirement)
        get_target_property(tmp_options requirement::component INTERFACE_LINK_OPTIONS)
        message("component options: ${tmp_options}")
    """)

    t.save({"conanfile.py": conanfile, "CMakeLists.txt": cmakelists})
    t.run("create . --build missing -s build_type=Release")
    assert ('component options: '
            '$<$<CONFIG:Release>:'
            '$<$<STREQUAL:$<TARGET_PROPERTY:TYPE>,SHARED_LIBRARY>:-Wl,-link1;-Wl,-link2>;'
            '$<$<STREQUAL:$<TARGET_PROPERTY:TYPE>,MODULE_LIBRARY>:-Wl,-link1;-Wl,-link2>;'
            '$<$<STREQUAL:$<TARGET_PROPERTY:TYPE>,EXECUTABLE>:>>') in t.out
    # NOTE: If there is no "conan install -s build_type=Debug", the properties won't contain the
    #       <CONFIG:Debug>


@pytest.mark.tool_cmake
def test_cmake_add_subdirectory():
    """https://github.com/conan-io/conan/issues/11743
       https://github.com/conan-io/conan/issues/11755"""

    t = TestClient()
    boost = textwrap.dedent("""
        from conan import ConanFile

        class Consumer(ConanFile):
            name = "boost"
            version = "1.0"

            def package_info(self):
                self.cpp_info.set_property("cmake_file_name", "Boost")
                self.cpp_info.components["A"].system_libs = ["A_1", "A_2"]
                self.cpp_info.components["B"].system_libs = ["B_1", "B_2"]
    """)
    t.save({"conanfile.py": boost})
    t.run("create .")
    conanfile = textwrap.dedent("""
            from conans import ConanFile
            from conan.tools.cmake import CMake, cmake_layout

            class Consumer(ConanFile):
                name = "consumer"
                version = "0.1"
                requires = "boost/1.0"
                generators = "CMakeDeps", "CMakeToolchain"
                settings = "os", "arch", "compiler", "build_type"

                def layout(self):
                    cmake_layout(self)

                def build(self):
                    cmake = CMake(self)
                    cmake.configure()
        """)

    cmakelists = textwrap.dedent("""
            cmake_minimum_required(VERSION 3.15)
            project(hello CXX)
            find_package(Boost CONFIG)
            add_subdirectory(src)

    """)
    sub_cmakelists = textwrap.dedent("""
            find_package(Boost REQUIRED COMPONENTS exception headers)

            message("AGGREGATED LIBS: ${Boost_LIBRARIES}")
            get_target_property(tmp boost::boost INTERFACE_LINK_LIBRARIES)
            message("AGGREGATED LINKED: ${tmp}")

            get_target_property(tmp boost::B INTERFACE_LINK_LIBRARIES)
            message("BOOST_B LINKED: ${tmp}")

            get_target_property(tmp boost::A INTERFACE_LINK_LIBRARIES)
            message("BOOST_A LINKED: ${tmp}")

            get_target_property(tmp boost_boost_B_DEPS_TARGET INTERFACE_LINK_LIBRARIES)
            message("BOOST_B_DEPS LINKED: ${tmp}")

            get_target_property(tmp boost_boost_A_DEPS_TARGET INTERFACE_LINK_LIBRARIES)
            message("BOOST_A_DEPS LINKED: ${tmp}")

    """)

    t.save({"conanfile.py": conanfile,
            "CMakeLists.txt": cmakelists, "src/CMakeLists.txt": sub_cmakelists})
    t.run("install .")
    # only doing the configure failed before #11743 fix
    t.run("build .")
    # The boost::boost target has linked the two components
    assert "AGGREGATED LIBS: boost::boost" in t.out
    assert "AGGREGATED LINKED: boost::B;boost::A" in t.out
    assert "BOOST_B LINKED: $<$<CONFIG:Release>:>;$<$<CONFIG:Release>:>;boost_boost_B_DEPS_TARGET" in t.out
    assert "BOOST_A LINKED: $<$<CONFIG:Release>:>;$<$<CONFIG:Release>:>;boost_boost_A_DEPS_TARGET" in t.out
    assert "BOOST_B_DEPS LINKED: $<$<CONFIG:Release>:>;$<$<CONFIG:Release>:B_1;B_2>" in t.out
    assert "BOOST_A_DEPS LINKED: $<$<CONFIG:Release>:>;$<$<CONFIG:Release>:A_1;A_2>;" in t.out



import os
import platform
import pytest
import six

from conan.tools.files import rename
from conans.test.utils.mocks import ConanFileMock
from conans.test.utils.tools import temp_folder, save_files


@pytest.mark.skipif(six.PY2, reason="only Py3")
def test_rename_file():
    conanfile = ConanFileMock()
    tmp = temp_folder()
    save_files(tmp, {"file.txt": ""})
    old_path = os.path.join(tmp, "file.txt")
    new_path = os.path.join(tmp, "kk.txt")
    rename(conanfile, old_path, new_path)
    assert not os.path.exists(old_path)
    assert os.path.exists(new_path)


@pytest.mark.skipif(six.PY2, reason="only Py3")
@pytest.mark.skipif(platform.system() != "Windows", reason="Robocopy only exists in Windows")
def test_rename_folder_robocopy():
    conanfile = ConanFileMock()
    tmp = temp_folder()
    old_folder = os.path.join(tmp, "old_folder")
    os.mkdir(old_folder)
    new_folder = os.path.join(tmp, "new_folder")
    rename(conanfile, old_folder, new_folder)
    assert not os.path.exists(old_folder)
    assert os.path.exists(new_folder)



import os
import platform
import unittest

import pytest

from conans.client.generators.text import TXTGenerator
from conans.paths import BUILD_INFO, BUILD_INFO_CMAKE, CONANFILE
from conans.test.assets.cpp_test_files import cpp_hello_conan_files
from conans.test.utils.tools import TestClient, TestServer
from conans.util.files import load


@pytest.mark.slow
class DiamondTest(unittest.TestCase):

    def setUp(self):
        test_server = TestServer([],  # write permissions
                                 users={"lasote": "mypass"})  # exported users and passwords
        self.servers = {"default": test_server}

    def test_diamond_cmake(self):
        self.client = TestClient(servers=self.servers, users={"default": [("lasote", "mypass")]})
        self._run(use_cmake=True, language=1)

    def test_diamond_cmake_targets(self):
        self.client = TestClient(servers=self.servers, users={"default": [("lasote", "mypass")]})
        self._run(use_cmake=True, cmake_targets=True)

    def test_diamond_default(self):
        self.client = TestClient(servers=self.servers, users={"default": [("lasote", "mypass")]},
                                 path_with_spaces=False)
        self._run(use_cmake=False)

    def _export(self, name, version=None, deps=None, use_cmake=True, cmake_targets=False):
        files = cpp_hello_conan_files(name, version, deps, need_patch=True, use_cmake=use_cmake,
                                      cmake_targets=cmake_targets, with_exe=False)
        self.client.save(files, clean_first=True)
        self.client.run("export . lasote/stable")

    def _check_individual_deps(self):
        self.assertIn("INCLUDE [", self.client.out)
        self.assertIn("/data/Hello0/0.1/lasote/stable", self.client.out)
        build_file = os.path.join(self.client.current_folder, BUILD_INFO)
        content = load(build_file)
        cmakebuildinfo = load(os.path.join(self.client.current_folder, BUILD_INFO_CMAKE))
        self.assertIn("set(CONAN_LIBS helloHello3 helloHello1 helloHello2 helloHello0",
                      cmakebuildinfo)
        self.assertIn("set(CONAN_DEPENDENCIES Hello3 Hello1 Hello2 Hello0)", cmakebuildinfo)
        deps_cpp_info, _, _, _ = TXTGenerator.loads(content)
        self.assertEqual(len(deps_cpp_info.include_paths), 4)
        for dep in ("Hello3", "Hello2", "Hello1", "Hello0"):
            self.assertEqual(len(deps_cpp_info[dep].include_paths), 1)
            self.assertEqual(len(deps_cpp_info[dep].lib_paths), 1)
            self.assertEqual(deps_cpp_info[dep].libs, ["hello%s" % dep])
        build_file = os.path.join(self.client.current_folder, BUILD_INFO_CMAKE)
        content = load(build_file)
        for dep in ("Hello3", "Hello2", "Hello1", "Hello0"):
            self.assertEqual(len(deps_cpp_info[dep].include_paths), 1)
            self.assertIn("set(CONAN_INCLUDE_DIRS_%s " % dep.upper(), content)
            self.assertIn("set(CONAN_LIBS_%s hello%s)" % (dep.upper(), dep), content)

    def _run(self, use_cmake=True, cmake_targets=False, language=0):

        if not use_cmake and platform.system() == "SunOS":
            return  # If is using sun-cc the gcc generator doesn't work

        self._export("Hello0", "0.1", use_cmake=use_cmake, cmake_targets=cmake_targets)
        self._export("Hello1", "0.1", ["Hello0/0.1@lasote/stable"], use_cmake=use_cmake,
                     cmake_targets=cmake_targets)
        self._export("Hello2", "0.1", ["Hello0/0.1@lasote/stable"], use_cmake=use_cmake,
                     cmake_targets=cmake_targets)
        self._export("Hello3", "0.1", ["Hello1/0.1@lasote/stable", "Hello2/0.1@lasote/stable"],
                     use_cmake=use_cmake, cmake_targets=cmake_targets)

        files3 = cpp_hello_conan_files("Hello4", "0.1", ["Hello3/0.1@lasote/stable"],
                                       language=language, use_cmake=use_cmake,
                                       cmake_targets=cmake_targets)

        # Add some stuff to base project conanfile to test further the individual
        # flags in build_info (txt, cmake) files
        content = files3[CONANFILE]
        content = content.replace("generators =", 'generators = "txt",')
        content = content.replace("def build(self):",
                                  "def build(self):\n"
                                  "        self.output.info('INCLUDE %s' "
                                  "% list(self.deps_cpp_info['Hello0'].include_paths))")
        files3[CONANFILE] = content
        self.client.save(files3)

        self.client.run("install . --build missing")
        if use_cmake:
            if cmake_targets:
                self.assertIn("Conan: Using cmake targets configuration", self.client.out)
                self.assertNotIn("Conan: Using cmake global configuration", self.client.out)
            else:
                self.assertIn("Conan: Using cmake global configuration", self.client.out)
                self.assertNotIn("Conan: Using cmake targets configuration", self.client.out)
        self.client.run("build .")
        self._check_individual_deps()

        def check_run_output(client):
            command = os.sep.join([".", "bin", "say_hello"])
            client.run_command(command)
            if language == 0:
                self.assertEqual(['Hello Hello4', 'Hello Hello3', 'Hello Hello1', 'Hello Hello0',
                                  'Hello Hello2', 'Hello Hello0'],
                                 str(client.out).splitlines()[-6:])
            else:
                self.assertEqual(['Hola Hello4', 'Hola Hello3', 'Hola Hello1', 'Hola Hello0',
                                  'Hola Hello2', 'Hola Hello0'],
                                 str(client.out).splitlines()[-6:])

        check_run_output(self.client)

        # Try to upload and reuse the binaries
        self.client.run("upload Hello* --all --confirm")
        self.assertEqual(str(self.client.out).count("Uploading package"), 4)

        # Reuse in another client
        client2 = TestClient(servers=self.servers, users={"default": [("lasote", "mypass")]},
                             path_with_spaces=use_cmake)
        files3[CONANFILE] = files3[CONANFILE].replace("generators =", 'generators = "txt",')
        client2.save(files3)
        client2.run("install .")
        client2.run("build .")

        self.assertNotIn("libhello0.a", client2.out)
        self.assertNotIn("libhello1.a", client2.out)
        self.assertNotIn("libhello2.a", client2.out)
        self.assertNotIn("libhello3.a", client2.out)
        check_run_output(client2)



import unittest

from conans.paths import CONANFILE
from conans.test.utils.tools import TestClient, GenConanfile


class VersionRangesErrorTest(unittest.TestCase):
    def test_verbose_version(self):
        client = TestClient()
        conanfile = GenConanfile().with_name("MyPkg").with_version("0.1")\
                                  .with_require("MyOtherPkg/[~0.1]@user/testing")
        client.save({CONANFILE: str(conanfile)})
        client.run("install . --build", assert_error=True)
        self.assertIn("from requirement 'MyOtherPkg/[~0.1]@user/testing'", client.out)



import unittest

from conans.test.utils.tools import TestClient, GenConanfile


class DeprecatedTestCase(unittest.TestCase):
    def test_no_deprecated(self):
        t = TestClient()
        t.save({'taskflow.py': GenConanfile("cpp-taskflow", "1.0").with_deprecated("''")})
        t.run("create taskflow.py")

        self.assertNotIn("Please, consider changing your requirements.", t.out)

    def test_deprecated_simple(self):
        t = TestClient()
        t.save({'taskflow.py': GenConanfile("cpp-taskflow", "1.0").with_deprecated("True")})
        t.run("create taskflow.py")

        self.assertIn("cpp-taskflow/1.0: WARN: Recipe 'cpp-taskflow/1.0' is deprecated. "
                      "Please, consider changing your requirements.", t.out)

        t.run("create taskflow.py conan/stable")
        self.assertIn("cpp-taskflow/1.0@conan/stable: WARN: Recipe 'cpp-taskflow/1.0@conan/stable' "
                      "is deprecated. Please, consider changing your requirements.", t.out)

    def test_deprecated_with(self):
        t = TestClient()
        t.save({'taskflow.py': GenConanfile("cpp-taskflow", "1.0").with_deprecated('"taskflow"')})
        t.run("create taskflow.py")

        self.assertIn("cpp-taskflow/1.0: WARN: Recipe 'cpp-taskflow/1.0' is deprecated in "
                      "favor of 'taskflow'. Please, consider changing your requirements.", t.out)

        t.run("create taskflow.py conan/stable")
        self.assertIn("cpp-taskflow/1.0@conan/stable: WARN: Recipe 'cpp-taskflow/1.0@conan/stable' "
                      "is deprecated in favor of 'taskflow'. Please, consider "
                      "changing your requirements.", t.out)



import os
from conans.client.conan_api import (ConanApp as ConanApp_co, ConanAPIV1, api_method, _get_conanfile_path, 
                                    _make_abs_path, 
                                    default_manifest_folder as default_manifest_folder_co,
                                    ProfileData as ProfileData_co, get_graph_info, _parse_manifests_arguments
                                    )
from conans.errors import (ConanException, RecipeNotFoundException,
                           PackageNotFoundException, NoRestV2Available, NotFoundException)
from sense2.client.conanfile.deb import run_deb_method
from conans.client.recorder.upload_recoder import UploadRecorder
from sense2.client.cmd.uploader_deb import CmdUploadDeb
from conans.client.recorder.action_recorder import ActionRecorder
from conans.client.cmd.export import cmd_export
from conans.model.ref import ConanFileReference
from conans.model.graph_lock import GraphLockFile
from sense2.client.cmd.create import create
from conans.client.graph.graph_manager import GraphManager
from conans.client.loader import ConanFileLoader


default_manifest_folder = default_manifest_folder_co

class ProfileData(ProfileData_co):
    def __bool__(self):
        return super().__bool__()
    __nonzero__ = __bool__

class Sense2App(ConanApp_co):
    def __init__(self, cache_folder, user_io, http_requester=None, runner=None, quiet_output=None):
        super().__init__(cache_folder, user_io, http_requester, runner, quiet_output)
        
        self.loader = ConanFileLoader(self.runner, self.out, self.python_requires,
                                      self.generator_manager, self.pyreq_loader, self.requester)

        self.graph_manager = GraphManager(self.out, self.cache, self.remote_manager, self.loader,
                                          self.proxy, self.range_resolver, self.binaries_analyzer)


class Sense2APIV1(ConanAPIV1):
    
    def create_app(self, quiet_output=None):
        self.app = Sense2App(self.cache_folder, self.user_io, self.http_requester,
                            self.runner, quiet_output=quiet_output)
    
    @api_method
    def create(self, conanfile_path, name=None, version=None, user=None, channel=None,
               profile_names=None, settings=None,
               options=None, env=None, test_folder=None, not_export=False,
               build_modes=None,
               keep_source=False, keep_build=False, verify=None,
               manifests=None, manifests_interactive=None,
               remote_name=None, update=False, cwd=None, test_build_folder=None,
               lockfile=None, lockfile_out=None, ignore_dirty=False, profile_build=None,
               is_build_require=False, conf=None, require_overrides=None):
        """
        API method to create a conan package

        test_folder default None   - looks for default 'test' or 'test_package' folder),
                                    string - test_folder path
                                    False  - disabling tests
        """
        profile_host = ProfileData(profiles=profile_names, settings=settings, options=options,
                                   env=env, conf=conf)
        cwd = cwd or os.getcwd()
        recorder = ActionRecorder()
        try:
            conanfile_path = _get_conanfile_path(conanfile_path, cwd, py=True)

            remotes = self.app.load_remotes(remote_name=remote_name, update=update)
            lockfile = _make_abs_path(lockfile, cwd) if lockfile else None
            graph_info = get_graph_info(profile_host, profile_build, cwd, None,
                                        self.app.cache, self.app.out, lockfile=lockfile)

            # Make sure keep_source is set for keep_build
            keep_source = keep_source or keep_build
            new_ref = cmd_export(self.app, conanfile_path, name, version, user, channel, keep_source,
                                 not not_export, graph_lock=graph_info.graph_lock,
                                 ignore_dirty=ignore_dirty)

            self.app.range_resolver.clear_output()  # invalidate version range output

            # The new_ref contains the revision
            # To not break existing things, that they used this ref without revision
            ref = new_ref.copy_clear_rev()
            recorder.recipe_exported(new_ref)

            if build_modes is None:  # Not specified, force build the tested library
                build_modes = [ref.name]

            manifests = _parse_manifests_arguments(verify, manifests, manifests_interactive, cwd)
            manifest_folder, manifest_interactive, manifest_verify = manifests

            # FIXME: Dirty hack: remove the root for the test_package/conanfile.py consumer
            graph_info.root = ConanFileReference(None, None, None, None, validate=False)
            recorder.add_recipe_being_developed(ref)
            create(self.app, ref, graph_info, remotes, update, build_modes,
                   manifest_folder, manifest_verify, manifest_interactive, keep_build,
                   test_build_folder, test_folder, conanfile_path, recorder=recorder,
                   is_build_require=is_build_require, require_overrides=require_overrides)

            if lockfile_out:
                lockfile_out = _make_abs_path(lockfile_out, cwd)
                graph_lock_file = GraphLockFile(graph_info.profile_host, graph_info.profile_build,
                                                graph_info.graph_lock)
                graph_lock_file.save(lockfile_out)
            return recorder.get_info(self.app.config.revisions_enabled)

        except ConanException as exc:
            recorder.error = True
            exc.info = recorder.get_info(self.app.config.revisions_enabled)
            raise
    
    
    @api_method
    def install_deb(self, conanfile_path, info_folder=None, cwd=None):
        cwd = cwd or os.getcwd()
        conanfile_path = _get_conanfile_path(conanfile_path, cwd, py=True)
        info_folder = _make_abs_path(info_folder, cwd)

        if not os.path.exists(info_folder):
            raise ConanException("Specified info-folder doesn't exist")
        print("conan_api install_deb")
        # only infos if exist
        conanfile = self.app.graph_manager.load_consumer_conanfile(conanfile_path, info_folder)
        conanfile.install_deb()


    @api_method
    def install_docker(self, conanfile_path, info_folder=None, cwd=None):
        cwd = cwd or os.getcwd()
        conanfile_path = _get_conanfile_path(conanfile_path, cwd, py=True)
        info_folder = _make_abs_path(info_folder, cwd)

        if not os.path.exists(info_folder):
            raise ConanException("Specified info-folder doesn't exist")
        print("conan_api install_docker")
        # only infos if exist
        conanfile = self.app.graph_manager.load_consumer_conanfile(conanfile_path, info_folder)
        conanfile.install_docker()

    @api_method
    def install_sense_system(self, conanfile_path, info_folder=None, cwd=None):
        cwd = cwd or os.getcwd()
        conanfile_path = _get_conanfile_path(conanfile_path, cwd, py=True)
        info_folder = _make_abs_path(info_folder, cwd)

        if not os.path.exists(info_folder):
            raise ConanException("Specified info-folder doesn't exist")
        print("conan_api install_sense_system")
        # only infos if exist
        conanfile = self.app.graph_manager.load_consumer_conanfile(conanfile_path, info_folder)
        conanfile.install_sense_system()
        
        
    @api_method
    def deb(self, path, build_folder, deb_folder, source_folder=None, install_folder=None,
                cwd=None):
        self.app.load_remotes()

        cwd = cwd or os.getcwd()
        conanfile_path = _get_conanfile_path(path, cwd, py=True)
        build_folder = _make_abs_path(build_folder, cwd)
        source_folder = _make_abs_path(source_folder, cwd, default=os.path.dirname(conanfile_path))
        install_folder = _make_abs_path(install_folder, cwd, default=build_folder)

        conanfile = self.app.graph_manager.load_consumer_conanfile(conanfile_path, install_folder,
                                                                   deps_info_required=True)
        
        default_deb_folder = os.path.join(build_folder, "deb")
        deb_folder = _make_abs_path(deb_folder, cwd, default=default_deb_folder)

        if hasattr(conanfile, "layout"):
            raise ConanException("The usage of the 'conan package' local method is disabled when "
                                 "using layout(). Use 'export-pkg' to test if the recipe is "
                                 "packaging the files correctly or use the cpp.info.local object "
                                 "if you are going to use this package as editable package.")
        else:
            conanfile.folders.set_base_build(build_folder)
            conanfile.folders.set_base_source(source_folder)
            conanfile.folders.set_base_deb(deb_folder)
            conanfile.folders.set_base_install(install_folder)

        run_deb_method(conanfile, None, self.app.hook_manager, conanfile_path, None,
                           copy_info=True)
        
    @api_method
    def upload_deb(self, pattern, package=None, remote_name=None, all_packages=False, confirm=False,
               retry=None, retry_wait=None, integrity_check=False, policy=None, query=None,
               parallel_upload=False):
        """ Uploads a package recipe and the generated binary packages to a specified remote
        """
        upload_recorder = UploadRecorder()
        uploader = CmdUploadDeb(self.app.cache, self.app.user_io, self.app.remote_manager,
                             self.app.loader, self.app.hook_manager, self.app.graph_manager)
        remotes = self.app.load_remotes(remote_name=remote_name)
        try:
            uploader.upload(pattern, remotes, upload_recorder, package, all_packages, confirm,
                            retry, retry_wait, integrity_check, policy, query=query,
                            parallel_upload=parallel_upload)
            return upload_recorder.get_info()
        except ConanException as exc:
            upload_recorder.error = True
            exc.info = upload_recorder.get_info()
            raise

    # @api_method
    # def set_source(self, conanfile_path, source_folder=None, info_folder=None, cwd=None):
    #     cwd = cwd or os.getcwd()
    #     conanfile_path = _get_conanfile_path(conanfile_path, cwd, py=True)
    #     source_folder = _make_abs_path(source_folder, cwd)
    #     info_folder = _make_abs_path(info_folder, cwd)


    #     if not os.path.exists(info_folder):
    #         raise ConanException("Specified info-folder doesn't exist")
    #     print("conan_api set_source")
    #     # only infos if exist
    #     conanfile = self.app.graph_manager.load_consumer_conanfile(conanfile_path, info_folder)
    #     conanfile.exports_sources = os.path.join(source_folder, "*")
    #     print("set exports_sources: ", conanfile.exports_sources)

        
Sense2 = Sense2APIV1



import os
import textwrap
import unittest

from parameterized import parameterized

from conans.test.utils.tools import TestClient
from conans.util.files import mkdir


class SetVersionNameTest(unittest.TestCase):

    @parameterized.expand([("", ), ("@user/channel", )])
    def test_set_version_name(self, user_channel):
        client = TestClient()
        conanfile = textwrap.dedent("""
            from conans import ConanFile
            class Lib(ConanFile):
                def set_name(self):
                    self.name = "pkg"
                def set_version(self):
                    self.version = "2.1"
            """)
        client.save({"conanfile.py": conanfile})
        client.run("export . %s" % user_channel)
        self.assertIn("pkg/2.1%s: A new conanfile.py version was exported" % user_channel,
                      client.out)
        # installing it doesn't break
        client.run("install pkg/2.1%s --build=missing" % (user_channel or "@"))
        self.assertIn("pkg/2.1%s:5ab84d6acfe1f23c4fae0ab88f26e3a396351ac9 - Build" % user_channel,
                      client.out)
        client.run("install pkg/2.1%s --build=missing" % (user_channel or "@"))
        self.assertIn("pkg/2.1%s:5ab84d6acfe1f23c4fae0ab88f26e3a396351ac9 - Cache" % user_channel,
                      client.out)

        # Local flow should also work
        client.run("install .")
        self.assertIn("conanfile.py (pkg/2.1): Installing package", client.out)

    def test_set_version_name_file(self):
        client = TestClient()
        conanfile = textwrap.dedent("""
            from conans import ConanFile, load
            class Lib(ConanFile):
                def set_name(self):
                    self.name = load("name.txt")
                def set_version(self):
                    self.version = load("version.txt")
            """)
        client.save({"conanfile.py": conanfile,
                     "name.txt": "pkg",
                     "version.txt": "2.1"})
        client.run("export . user/testing")
        self.assertIn("pkg/2.1@user/testing: A new conanfile.py version was exported", client.out)
        client.run("install pkg/2.1@user/testing --build=missing")
        self.assertIn("pkg/2.1@user/testing:5ab84d6acfe1f23c4fae0ab88f26e3a396351ac9 - Build",
                      client.out)
        client.run("install pkg/2.1@user/testing")
        self.assertIn("pkg/2.1@user/testing:5ab84d6acfe1f23c4fae0ab88f26e3a396351ac9 - Cache",
                      client.out)
        # Local flow should also work
        client.run("install .")
        self.assertIn("conanfile.py (pkg/2.1): Installing package", client.out)

    def test_set_version_name_errors(self):
        client = TestClient()
        conanfile = textwrap.dedent("""
            from conans import ConanFile
            class Lib(ConanFile):
                def set_name(self):
                    self.name = "pkg"
                def set_version(self):
                    self.version = "2.1"
            """)
        client.save({"conanfile.py": conanfile})
        client.run("export . other/1.1@user/testing", assert_error=True)
        self.assertIn("ERROR: Package recipe with name other!=pkg", client.out)
        client.run("export . 1.1@user/testing", assert_error=True)
        self.assertIn("ERROR: Package recipe with version 1.1!=2.1", client.out)
        # These are checked but match and don't conflict
        client.run("export . 2.1@user/testing")
        client.run("export . pkg/2.1@user/testing")

        # Local flow should also fail
        client.run("install . other/1.2@", assert_error=True)
        self.assertIn("ERROR: Package recipe with name other!=pkg", client.out)

    def test_set_version_name_only_not_cli(self):
        client = TestClient()
        conanfile = textwrap.dedent("""
            from conans import ConanFile
            class Lib(ConanFile):
                def set_name(self):
                    self.name = self.name or "pkg"
                def set_version(self):
                    self.version = self.version or "2.0"
            """)
        client.save({"conanfile.py": conanfile})
        client.run("export . other/1.1@user/testing")
        self.assertIn("other/1.1@user/testing: Exported", client.out)
        client.run("export . 1.1@user/testing")
        self.assertIn("pkg/1.1@user/testing: Exported", client.out)
        client.run("export . user/testing")
        self.assertIn("pkg/2.0@user/testing: Exported", client.out)

        # Local flow should also work
        client.run("install . other/1.2@")
        self.assertIn("conanfile.py (other/1.2)", client.out)
        client.run("install .")
        self.assertIn("conanfile.py (pkg/2.0)", client.out)

    def test_set_version_name_crash(self):
        client = TestClient()
        conanfile = textwrap.dedent("""
            from conans import ConanFile
            class Lib(ConanFile):
                def set_name(self):
                    self.name = error
            """)
        client.save({"conanfile.py": conanfile})
        client.run("export .", assert_error=True)
        self.assertIn("ERROR: conanfile.py: Error in set_name() method, line 5", client.out)
        self.assertIn("name 'error' is not defined", client.out)
        conanfile = textwrap.dedent("""
           from conans import ConanFile
           class Lib(ConanFile):
               def set_version(self):
                   self.version = error
           """)
        client.save({"conanfile.py": conanfile})
        client.run("export .", assert_error=True)
        self.assertIn("ERROR: conanfile.py: Error in set_version() method, line 5", client.out)
        self.assertIn("name 'error' is not defined", client.out)

    def test_set_version_recipe_folder(self):
        client = TestClient()
        conanfile = textwrap.dedent("""
            import os
            from conans import ConanFile, load
            class Lib(ConanFile):
                name = "pkg"
                def set_version(self):
                    self.version = load(os.path.join(self.recipe_folder, "version.txt"))
            """)
        client.save({"conanfile.py": conanfile,
                     "version.txt": "2.1"})
        mkdir(os.path.join(client.current_folder, "build"))
        with client.chdir("build"):
            client.run("export .. user/testing")
            self.assertIn("pkg/2.1@user/testing: A new conanfile.py version was exported",
                          client.out)

        # This is reusable with python_requires too
        reuse = textwrap.dedent("""
            from conans import python_requires
            tool = python_requires("pkg/2.1@user/testing")
            class Consumer(tool.Lib):
                name = "consumer"
            """)
        client2 = TestClient(cache_folder=client.cache_folder)
        client2.save({"conanfile.py": reuse,
                      "version.txt": "8.3"})
        client2.run("export .")
        self.assertIn("consumer/8.3: A new conanfile.py version was exported", client2.out)

    def test_set_version_cwd(self):
        client = TestClient()
        conanfile = textwrap.dedent("""
            import os
            from conans import ConanFile, load
            class Lib(ConanFile):
                name = "pkg"
                def set_version(self):
                    self.version = load("version.txt")
            """)
        client.save({"conanfile.py": conanfile})
        mkdir(os.path.join(client.current_folder, "build"))
        with client.chdir("build"):
            client.save({"version.txt": "2.1"}, clean_first=True)
            client.run("export .. ")
            self.assertIn("pkg/2.1: A new conanfile.py version was exported", client.out)



import os
import platform
import sys
import textwrap

import pytest

from conans.test.assets.sources import gen_function_cpp
from conans.test.utils.tools import TestClient


@pytest.mark.tool_meson
@pytest.mark.skipif(sys.version_info.major == 2, reason="Meson not supported in Py2")
@pytest.mark.skipif(platform.system() != "Windows", reason="requires Windows")
def test_cross_x86():
    conanfile_py = textwrap.dedent("""
        from conans import ConanFile, tools
        from conan.tools.meson import Meson, MesonToolchain


        class App(ConanFile):
            settings = "os", "arch", "compiler", "build_type"
            options = {"shared": [True, False], "fPIC": [True, False]}
            default_options = {"shared": False, "fPIC": True}

            def config_options(self):
                if self.settings.os == "Windows":
                    del self.options.fPIC

            def layout(self):
                self.folders.build = "build"

            def generate(self):
                tc = MesonToolchain(self, backend='vs')
                tc.generate()

            def build(self):
                meson = Meson(self)
                meson.configure()
                meson.build()
        """)
    meson_build = textwrap.dedent("""
        project('tutorial', 'cpp')
        executable('demo', 'main.cpp')
        """)
    main_cpp = gen_function_cpp(name="main")
    client = TestClient()
    client.save({"conanfile.py": conanfile_py,
                 "meson.build": meson_build,
                 "main.cpp": main_cpp})
    client.run("install .")
    content = client.load("conan_meson_native.ini")
    assert "backend = 'vs'" in content
    client.run("build .")
    assert "Auto detected Visual Studio backend" in client.out
    client.run_command(os.path.join("build", "demo"))

    assert "main _M_X64 defined" in client.out
    assert "main _MSC_VER19" in client.out
    assert "main _MSVC_LANG2014" in client.out



from conan.tools.microsoft.visual import vcvars_command


def check_vs_runtime(artifact, client, vs_version, build_type, architecture="amd64",
                     static_runtime=False, subsystem=None):
    vcvars = vcvars_command(version=vs_version, architecture=architecture)
    normalized_path = artifact.replace("/", "\\")
    static = artifact.endswith(".a") or artifact.endswith(".lib")
    if not static:
        cmd = ('%s && dumpbin /nologo /dependents "%s"' % (vcvars, normalized_path))
        client.run_command(cmd)
        if subsystem:
            assert "KERNEL32.dll" in client.out
            if subsystem in ("mingw32", "mingw64"):
                assert "msvcrt.dll" in client.out
                if static_runtime:
                    assert "libstdc++-6.dll" not in client.out
                else:
                    assert "libstdc++-6.dll" in client.out
                if subsystem == "mingw32":
                    if static_runtime:
                        assert "libgcc_s_dw2-1.dll" not in client.out
                    else:
                        assert "libgcc_s_dw2-1.dll" in client.out
            elif subsystem == "msys2":
                assert "msys-2.0.dll" in client.out
                if static_runtime:
                    assert "msys-stdc++-6.dll" not in client.out
                else:
                    assert "msys-stdc++-6.dll" in client.out
            elif subsystem == "cygwin":
                assert "cygwin1.dll" in client.out
                if static_runtime:
                    assert "cygstdc++-6.dll" not in client.out
                else:
                    assert "cygstdc++-6.dll" in client.out
            elif subsystem == "ucrt64":
                assert "api-ms-win-crt-" in client.out
                if static_runtime:
                    assert "libstdc++-6.dll" not in client.out
                else:
                    assert "libstdc++-6.dll" in client.out
            elif subsystem == "clang64":
                assert "api-ms-win-crt-" in client.out
                if static_runtime:
                    assert "libunwind.dll" not in client.out
                    assert "libc++.dll" not in client.out
                else:
                    assert "libunwind.dll" in client.out
                    assert "libc++.dll" in client.out
            else:
                raise Exception("unknown {}".format(subsystem))
        elif static_runtime:
            assert "KERNEL32.dll" in client.out
            assert "MSVC" not in client.out
            assert "VCRUNTIME" not in client.out
        else:
            assert "KERNEL32.dll" in client.out
            if build_type == "Debug":
                assert "ucrtbased" in client.out
            else:
                assert "api-ms-win-crt-" in client.out
            if vs_version in ["15", "16", "17"]:  # UCRT
                debug = "D" if build_type == "Debug" else ""
                assert "MSVCP140{}.dll".format(debug) in client.out
                assert "VCRUNTIME140{}.dll".format(debug) in client.out
            else:
                raise NotImplementedError()
    else:  # A static library cannot be checked the same
        client.run_command('{} && DUMPBIN /NOLOGO /DIRECTIVES "{}"'.format(vcvars, artifact))
        if build_type == "Debug":
            assert "RuntimeLibrary=MDd_DynamicDebug" in client.out
        else:
            assert "RuntimeLibrary=MD_DynamicRelease" in client.out


def check_exe_run(output, names, compiler, version, build_type, arch, cppstd, definitions=None,
                  cxx11_abi=None, subsystem=None):
    output = str(output)
    names = names if isinstance(names, list) else [names]

    for name in names:
        assert "{}: {}".format(name, build_type) in output
        if compiler == "msvc":
            if arch == "x86":
                assert "{} _M_IX86 defined".format(name) in output
            elif arch == "x86_64":
                assert "{} _M_X64 defined".format(name) in output
            elif arch == "armv8":
                assert "{} _M_ARM64 defined".format(name) in output
            else:
                assert arch is None, "checked don't know how to validate this architecture"

            if version:
                assert "{} _MSC_VER{}".format(name, version) in output
            if cppstd:
                assert "{} _MSVC_LANG20{}".format(name, cppstd) in output

        elif compiler in ["gcc", "clang", "apple-clang"]:
            if compiler == "gcc":
                assert "{} __GNUC__".format(name) in output
                assert "clang" not in output
                if version:  # FIXME: At the moment, the GCC version is not controlled, will change
                    major, minor = version.split(".")[0:2]
                    assert "{} __GNUC__{}".format(name, major) in output
                    assert "{} __GNUC_MINOR__{}".format(name, minor) in output
            elif compiler == "clang":
                assert "{} __clang_".format(name) in output
                if version:
                    major, minor = version.split(".")[0:2]
                    assert "{} __clang_major__{}".format(name, major) in output
                    assert "{} __clang_minor__{}".format(name, minor) in output
            elif compiler == "apple-clang":
                assert "{} __apple_build_version__".format(name) in output
                if version:
                    major, minor = version.split(".")[0:2]
                    assert "{} __apple_build_version__{}{}".format(name, major, minor) in output
            if arch == "x86":
                assert "{} __i386__ defined".format(name) in output
            elif arch == "x86_64":
                assert "{} __x86_64__ defined".format(name) in output
            elif arch == "armv8":
                assert "{} __aarch64__ defined".format(name) in output
            else:
                assert arch is None, "checked don't know how to validate this architecture"

            if cppstd:
                cppstd_value = {"98": "199711",
                                "11": "201103",
                                "14": "201402",
                                "17": "201703"}[cppstd]
                assert "{} __cplusplus{}".format(name, cppstd_value) in output

            if cxx11_abi is not None:
                assert "{} _GLIBCXX_USE_CXX11_ABI {}".format(name, cxx11_abi) in output

        if definitions:
            for k, v in definitions.items():
                assert "{}: {}".format(k, v) in output

        if subsystem:
            if subsystem == "msys2":
                assert "__MSYS__" in output
                assert "__CYGWIN__" in output
                assert "MINGW" not in output
            elif subsystem in ("mingw32", "mingw64"):
                assert "__MINGW32__" in output
                assert "__CYGWIN__" not in output
                assert "__MSYS__" not in output
                if subsystem == "mingw64":
                    assert "__MINGW64__" in output
                else:
                    assert "MING64" not in output
            elif subsystem == "cygwin":
                assert "__CYGWIN__" in output
                assert "__MINGW32__" not in output
                assert "__MINGW64__" not in output
                assert "__MSYS__" not in output
            else:
                raise Exception("unknown subsystem {}".format(subsystem))
        else:
            assert "CYGWIN" not in output
            assert "MINGW" not in output
            assert "MSYS" not in output



# coding=utf-8

import unittest

from parameterized import parameterized

from conans.test.utils.tools import TestClient, GenConanfile


class VersionRangeOverrideTestCase(unittest.TestCase):

    def setUp(self):
        self.t = TestClient()
        self.t.save({"libB/conanfile.py": GenConanfile(),
                     "libC/conanfile.py":
                         GenConanfile().with_require("libB/[<=2.0]@user/channel")})
        self.t.run("export libB libB/1.0@user/channel")
        self.t.run("export libB libB/2.0@user/channel")
        self.t.run("export libB libB/3.0@user/channel")
        self.t.run("export libC libC/1.0@user/channel")

        # Use the version range
        self.t.save({"conanfile.py": GenConanfile().with_require("libC/1.0@user/channel")})
        self.t.run("info . --only requires")
        self.assertIn("libB/2.0@user/channel", self.t.out)

    def test_override_with_fixed_version(self):
        # Override upstream version range with a fixed version
        self.t.save({"conanfile.py": GenConanfile().with_require("libB/3.0@user/channel")
                                                   .with_require("libC/1.0@user/channel")})
        self.t.run("info . --only requires")
        self.assertIn("libB/3.0@user/channel", self.t.out)
        self.assertIn("WARN: libC/1.0@user/channel: requirement libB/[<=2.0]@user/channel overridden"
                      " by your conanfile to libB/3.0@user/channel", self.t.out)

    def test_override_using_version_range(self):
        # Override upstream version range with a different (narrower) version range
        self.t.save({"conanfile.py": GenConanfile().with_require("libB/[<2.x]@user/channel")
                                                   .with_require("libC/1.0@user/channel")})
        self.t.run("info . --only requires")
        self.assertIn("libB/1.0@user/channel", self.t.out)
        self.assertIn("WARN: libC/1.0@user/channel: requirement libB/[<=2.0]@user/channel overridden"
                      " by your conanfile to libB/1.0@user/channel", self.t.out)
        self.assertIn("Version range '<2.x' required by 'conanfile.py' resolved to"
                      " 'libB/1.0@user/channel' in local cache", self.t.out)
        self.assertIn("Version range '<=2.0' required by 'libC/1.0@user/channel' valid for"
                      " downstream requirement 'libB/1.0@user/channel'", self.t.out)

    def test_override_version_range_outside(self):
        # Override upstream version range with a different (non intersecting) version range
        self.t.save({"conanfile.py": GenConanfile().with_require("libB/[>2.x]@user/channel")
                                                   .with_require("libC/1.0@user/channel")})
        self.t.run("info . --only requires", assert_error=True)
        self.assertIn("WARN: libC/1.0@user/channel: requirement libB/[<=2.0]@user/channel overridden"
                      " by your conanfile to libB/3.0@user/channel", self.t.out)
        self.assertIn("ERROR: Version range '<=2.0' required by 'libC/1.0@user/channel' not valid"
                      " for downstream requirement 'libB/3.0@user/channel'", self.t.out)


class VersionRangeOverrideFailTestCase(unittest.TestCase):

    @parameterized.expand([(True,), (False,)])
    def test(self, host_context):
        # https://github.com/conan-io/conan/issues/7864
        t = TestClient()
        t.save({"conanfile.py": GenConanfile()})
        t.run("create . gtest/1.8.0@PORT/stable")
        t.run("create . gtest/1.8.1@bloomberg/stable")

        t.save({"conanfile.py": GenConanfile().with_require("gtest/1.8.0@PORT/stable")})
        t.run("create . intermediate/1.0@PORT/stable")

        if host_context:
            t.save({"conanfile.py": GenConanfile().with_requires("intermediate/1.0@PORT/stable")
                   .with_build_requirement("gtest/1.8.0@PORT/stable", force_host_context=True)})
        else:
            # WARNING: This test will fail in Conan 2.0, because build_requires will be by default
            # in the build-context and do not conflict
            t.save({"conanfile.py": GenConanfile().with_requires("intermediate/1.0@PORT/stable")
                   .with_build_requires("gtest/1.8.0@PORT/stable")})
        t.run("create . scubaclient/1.6@PORT/stable")

        # IMPORTANT: We need to override the private build-require in the profile too,
        # otherwise it will conflict, as it will not be overriden by regular requires
        t.save({"conanfile.py": GenConanfile().with_requires("gtest/1.8.1@bloomberg/stable",
                                                             "scubaclient/1.6@PORT/stable"),
                "myprofile": "[build_requires]\ngtest/1.8.1@bloomberg/stable"})

        t.run("lock create conanfile.py --build -pr=myprofile")
        lock = t.load("conan.lock")
        self.assertIn("gtest/1.8.1@bloomberg/stable", lock)
        self.assertNotIn("gtest/1.8.0@PORT/stable", lock)



import platform
import textwrap
import unittest
from textwrap import dedent

import pytest

from conans.paths import CONANFILE
from conans.test.assets.autotools import gen_makefile_am, gen_configure_ac
from conans.test.utils.tools import TestClient
from conans.test.assets.cpp_test_files import cpp_hello_conan_files


mylibh = '''
double mean(double a, double b);
'''
conanfile = '''
from conans import ConanFile

class ConanMeanLib(ConanFile):
    name = "Mean"
    version = "0.1"
    exports_sources = "mean*"
    settings = "os", "compiler", "build_type", "arch"
    options = {"activate_define": [True, False], "optimize": [True, False]}
    default_options = """activate_define=True
optimize=False
"""
    generators = "gcc"

    def build(self):
        self.output.info("Running source!")
        self.run("c++ -c -o mean.o @conanbuildinfo.gcc mean.cpp")
        self.run("ar rcs libmean.a mean.o")

    def package(self):
        self.copy("*.h", dst="include")
        self.copy("*.a", dst="lib")

    def package_info(self):
        self.cpp_info.libs.append("mean")
        if self.options.activate_define:
            self.cpp_info.defines.append("ACTIVE_VAR=1")
        if self.options.optimize:
            self.cpp_info.cflags.append("-O2")
'''

mylib = '''
double mean(double a, double b) {
  return (a+b) / 2;
}
'''

example = '''
#include "mean.h"
#include <iostream>

int main(){
    std::cout << mean(10, 20) << std::endl;
    #ifdef ACTIVE_VAR
    std::cout << "Active var!!!" << std::endl;
    #endif
}
    '''


class BuildEnvironmenTest(unittest.TestCase):

    @pytest.mark.skipif(platform.system() != "Linux", reason="Requires Linux")
    @pytest.mark.tool_autotools
    def test_use_build_virtualenv(self):
        client = TestClient(path_with_spaces=False)
        client.save({CONANFILE: conanfile, "mean.cpp": mylib, "mean.h": mylibh})
        client.run("export . lasote/stable")

        makefile_am = gen_makefile_am(main="main", main_srcs="main.cpp")
        configure_ac = gen_configure_ac()

        reuse_conanfile = textwrap.dedent('''
            import platform
            from conans import ConanFile

            class ConanReuseLib(ConanFile):
                requires = "Mean/0.1@lasote/stable"
                generators = "virtualbuildenv"
                settings = "os", "compiler", "build_type", "arch"
                exports_sources = "*"

                def build(self):
                    self.run("aclocal")
                    self.run("autoconf")
                    self.run("automake --add-missing --foreign")
                    self.run("ls")
                    self.run("bash -c 'source activate_build.sh && ./configure'")
                    self.run("bash -c 'source activate_build.sh && make'")
                    self.run("./main")
            ''')
        client.save({CONANFILE: reuse_conanfile,
                     "Makefile.am": makefile_am,
                     "configure.ac": configure_ac,
                     "main.cpp": example})
        client.run("install . --build missing")
        client.run("build .")
        self.assertIn("15", client.out)

    @pytest.mark.skipif(platform.system() != "Windows", reason="Requires windows")
    def test_use_build_virtualenv_windows(self):
        files = cpp_hello_conan_files("hello", "0.1",  use_cmake=False, with_exe=False)
        client = TestClient(path_with_spaces=False)
        client.save(files)
        client.run("create . user/testing")
        files = cpp_hello_conan_files("hello2", "0.1", deps=["hello/0.1@user/testing"],
                                      use_cmake=False, with_exe=False)
        client.save(files, clean_first=True)
        client.run("create . user/testing")

        reuse_conanfile = dedent('''
            from conans import ConanFile

            class ConanReuseLib(ConanFile):
                requires = "hello2/0.1@user/testing"
                generators = "virtualbuildenv"
                settings = "os", "compiler", "build_type", "arch"

                def build(self):
                    self.run('activate_build.bat && cl /c /EHsc hello.cpp')
                    self.run('activate_build.bat && lib hello.obj -OUT:helloapp.lib')
                    self.run('activate_build.bat && cl /EHsc main.cpp helloapp.lib')
            ''')

        reuse = cpp_hello_conan_files("app", "0.1", deps=["hello2/0.1@user/testing"],
                                      use_cmake=False)
        reuse["conanfile.py"] = reuse_conanfile
        client.save(reuse, clean_first=True)
        client.run("install .")
        client.run("build .")
        client.run_command("main.exe")
        self.assertIn("Hello app;Hello hello2;Hello hello", ";".join(str(client.out).splitlines()))



import textwrap
import platform


import pytest

from conans.test.assets.cmake import gen_cmakelists
from conans.test.assets.sources import gen_function_cpp
from conans.test.utils.tools import TestClient


@pytest.mark.skipif(platform.system() != "Darwin", reason="Only OSX")
@pytest.mark.parametrize("op_system", ["Macos", "iOS"])
def test_m1(op_system):
    os_version = "os.version=12.0" if op_system == "iOS" else ""
    os_sdk = "" if op_system == "Macos" else "os.sdk=iphoneos"
    profile = textwrap.dedent("""
        include(default)
        [settings]
        os={}
        {}
        {}
        arch=armv8
    """.format(op_system, os_sdk, os_version))

    client = TestClient(path_with_spaces=False)
    client.save({"m1": profile}, clean_first=True)
    client.run("new hello/0.1 --template=cmake_lib")
    client.run("create . --profile:build=default --profile:host=m1 -tf None")

    main = gen_function_cpp(name="main", includes=["hello"], calls=["hello"])
    custom_content = 'message("CMAKE_SYSTEM_NAME: ${CMAKE_SYSTEM_NAME}") \n' \
                     'message("CMAKE_SYSTEM_PROCESSOR: ${CMAKE_SYSTEM_PROCESSOR}") \n'
    cmakelists = gen_cmakelists(find_package=["hello"], appname="main", appsources=["main.cpp"],
                                custom_content=custom_content)

    conanfile = textwrap.dedent("""
        from conans import ConanFile
        from conan.tools.cmake import CMake, cmake_layout

        class TestConan(ConanFile):
            requires = "hello/0.1"
            settings = "os", "compiler", "arch", "build_type"
            exports_sources = "CMakeLists.txt", "main.cpp"
            generators = "CMakeDeps", "CMakeToolchain"

            def layout(self):
                cmake_layout(self)

            def build(self):
                cmake = CMake(self)
                cmake.configure()
                cmake.build()
        """)

    client.save({"conanfile.py": conanfile,
                 "CMakeLists.txt": cmakelists,
                 "main.cpp": main,
                 "m1": profile}, clean_first=True)
    client.run("install . --profile:build=default --profile:host=m1")
    client.run("build .")
    system_name = 'Darwin' if op_system == 'Macos' else 'iOS'
    assert "CMAKE_SYSTEM_NAME: {}".format(system_name) in client.out
    assert "CMAKE_SYSTEM_PROCESSOR: arm64" in client.out
    main_path = "./build/Release/main.app/main" if op_system == "iOS" \
        else "./build/Release/main"
    client.run_command("lipo -info {}".format(main_path))
    assert "Non-fat file" in client.out
    assert "is architecture: arm64" in client.out
    client.run_command(f"vtool -show-build {main_path}")

    if op_system == "Macos":
        assert "platform MACOS"
    elif op_system == "iOS":
        assert "platform IOS"
        assert "minos 12.0"


import textwrap

from conans.client.graph.graph import CONTEXT_BUILD, CONTEXT_HOST
from conans.model.profile import Profile
from conans.test.integration.graph.core.cross_build._base_test_case import CrossBuildingBaseTestCase


class NoWayBackToHost(CrossBuildingBaseTestCase):

    application = textwrap.dedent("""
        from conans import ConanFile

        class Application(ConanFile):
            name = "app"
            version = "testing"

            settings = "os"

            def build_requirements(self):
                self.build_requires("protoc/testing@user/channel")
                self.build_requires("gtest/testing@user/channel", force_host_context=True)

            def build(self):
                self.output.info(">> settings.os:".format(self.settings.os))
    """)

    protoc = CrossBuildingBaseTestCase.protoc_tpl.render(
        build_requires=((CrossBuildingBaseTestCase.cmake_ref, False),
                        (CrossBuildingBaseTestCase.gtest_ref, True)))

    def setUp(self):
        super(NoWayBackToHost, self).setUp()
        self._cache_recipe(self.cmake_ref, self.cmake)
        self._cache_recipe(self.gtest_ref, self.gtest)
        self._cache_recipe(self.protobuf_ref, self.protobuf)
        self._cache_recipe(self.protoc_ref, self.protoc)
        self._cache_recipe(self.app_ref, self.application)

    def test_profile_access(self):
        profile_host = Profile()
        profile_host.settings["os"] = "Host"
        profile_host.process_settings(self.cache)

        profile_build = Profile()
        profile_build.settings["os"] = "Build"
        profile_build.process_settings(self.cache)

        deps_graph = self._build_graph(profile_host=profile_host, profile_build=profile_build,
                                       install=True)

        # - Application
        application = deps_graph.root.dependencies[0].dst
        self.assertEqual(application.conanfile.name, "app")
        self.assertEqual(application.context, CONTEXT_HOST)
        self.assertEqual(application.conanfile.settings.os, "Host")
        self.assertEqual(application.conanfile.settings_build.os, "Build")
        self.assertEqual(application.conanfile.settings_target, None)

        # - protoc
        protoc = application.dependencies[0].dst
        self.assertEqual(protoc.conanfile.name, "protoc")
        self.assertEqual(protoc.context, CONTEXT_BUILD)
        self.assertEqual(protoc.conanfile.settings.os, "Build")
        self.assertEqual(protoc.conanfile.settings_build.os, "Build")
        self.assertEqual(protoc.conanfile.settings_target.os, "Host")

        # - protoc/protobuf
        protobuf = protoc.dependencies[0].dst
        self.assertEqual(protobuf.conanfile.name, "protobuf")
        self.assertEqual(protobuf.context, CONTEXT_BUILD)
        self.assertEqual(protobuf.conanfile.settings.os, "Build")
        self.assertEqual(protobuf.conanfile.settings_build.os, "Build")
        self.assertEqual(protobuf.conanfile.settings_target.os, "Host")

        # - protoc/cmake
        cmake = protoc.dependencies[1].dst
        self.assertEqual(cmake.conanfile.name, "cmake")
        self.assertEqual(cmake.context, CONTEXT_BUILD)
        self.assertEqual(cmake.conanfile.settings.os, "Build")
        self.assertEqual(cmake.conanfile.settings_build.os, "Build")
        self.assertEqual(cmake.conanfile.settings_target.os, "Build")

        # - protoc/gtest
        protoc_gtest = protoc.dependencies[2].dst
        self.assertEqual(protoc_gtest.conanfile.name, "gtest")
        self.assertEqual(protoc_gtest.context, CONTEXT_BUILD)
        self.assertEqual(protoc_gtest.conanfile.settings.os, "Build")
        self.assertEqual(protoc_gtest.conanfile.settings_build.os, "Build")
        # We can't think about an scenario where a `build_require-host` should know about
        #   the target context. We are removing this information on purpose.
        self.assertEqual(protoc_gtest.conanfile.settings_target, None)

        # - gtest
        gtest = application.dependencies[1].dst
        self.assertEqual(gtest.conanfile.name, "gtest")
        self.assertEqual(gtest.context, CONTEXT_HOST)
        self.assertEqual(gtest.conanfile.settings.os, "Host")
        self.assertEqual(gtest.conanfile.settings_build.os, "Build")
        self.assertEqual(gtest.conanfile.settings_target, None)



import os
import textwrap

from conan.tools import CONAN_TOOLCHAIN_ARGS_FILE
from conan.tools.files.files import load_toolchain_args
from conans.test.utils.tools import TestClient


def test_bazel_namespace():
    client = TestClient()
    namespace = "somename"
    conanfile = textwrap.dedent("""
            from conans import ConanFile
            from conan.tools.google import BazelToolchain, Bazel

            class Conan(ConanFile):
                settings = "os", "arch", "compiler", "build_type"
                def generate(self):
                    bazel = BazelToolchain(self, namespace='{0}')
                    bazel.generate()
                def build(self):
                    bazel = Bazel(self, namespace='{0}')
                    self.output.info(bazel._bazel_config)
                    self.output.info(bazel._bazelrc_path)
            """.format(namespace))

    profile = textwrap.dedent("""
    include(default)
    [conf]
    tools.google.bazel:configs=["test_config"]
    tools.google.bazel:bazelrc_path=/path/to/bazelrc
    """)

    client.save({"test_profile": profile})

    client.save({"conanfile.py": conanfile})
    client.run("install . -pr test_profile")
    assert os.path.isfile(os.path.join(client.current_folder,
                                       "{}_{}".format(namespace, CONAN_TOOLCHAIN_ARGS_FILE)))
    content = load_toolchain_args(generators_folder=client.current_folder, namespace=namespace)
    bazel_config = content.get("bazel_configs")
    bazelrc_path = content.get("bazelrc_path")
    client.run("build .")
    assert bazel_config in client.out
    assert bazelrc_path in client.out


def test_autotools_namespace():
    client = TestClient()
    namespace = "somename"
    conanfile = textwrap.dedent("""
            from conans import ConanFile
            from conan.tools.gnu import AutotoolsToolchain, Autotools

            class Conan(ConanFile):
                settings = "os", "arch", "compiler", "build_type"
                def generate(self):
                    autotools = AutotoolsToolchain(self, namespace='{0}')
                    autotools.configure_args = ['a', 'b']
                    autotools.make_args = ['c', 'd']
                    autotools.generate()
                def build(self):
                    autotools = Autotools(self, namespace='{0}')
                    self.output.info(autotools._configure_args)
                    self.output.info(autotools._make_args)
            """.format(namespace))

    client.save({"conanfile.py": conanfile})
    client.run("install .")
    assert os.path.isfile(os.path.join(client.current_folder,
                                       "{}_{}".format(namespace, CONAN_TOOLCHAIN_ARGS_FILE)))
    content = load_toolchain_args(generators_folder=client.current_folder, namespace=namespace)
    at_configure_args = content.get("configure_args")
    at_make_args = content.get("make_args")
    client.run("build .")
    assert at_configure_args in client.out
    assert at_make_args in client.out


def test_multiple_toolchains_one_recipe():
    # https://github.com/conan-io/conan/issues/9376
    client = TestClient()
    namespaces = ["autotools", "bazel"]
    conanfile = textwrap.dedent("""
            from conans import ConanFile
            from conan.tools.gnu import AutotoolsToolchain, Autotools
            from conan.tools.google import BazelToolchain, Bazel

            class Conan(ConanFile):
                settings = "os", "arch", "compiler", "build_type"
                def generate(self):
                    autotools = AutotoolsToolchain(self, namespace='{0}')
                    autotools.configure_args = ['a', 'b']
                    autotools.make_args = ['c', 'd']
                    autotools.generate()
                    bazel = BazelToolchain(self, namespace='{1}')
                    bazel.generate()

                def build(self):
                    autotools = Autotools(self, namespace='{0}')
                    self.output.info(autotools._configure_args)
                    self.output.info(autotools._make_args)
                    bazel = Bazel(self, namespace='{1}')
                    self.output.info(bazel._bazel_config)
                    self.output.info(bazel._bazelrc_path)
            """.format(*namespaces))

    client.save({"conanfile.py": conanfile})

    profile = textwrap.dedent("""
    include(default)
    [conf]
    tools.google.bazel:configs=["test_config"]
    tools.google.bazel:bazelrc_path=/path/to/bazelrc
    """)

    client.save({"test_profile": profile})

    client.run("install . -pr test_profile")
    check_args = {
        "autotools": ["configure_args", "make_args"],
        "bazel": ["bazel_configs", "bazelrc_path"],
    }
    checks = []
    for namespace in namespaces:
        assert os.path.isfile(os.path.join(client.current_folder,
                                           "{}_{}".format(namespace, CONAN_TOOLCHAIN_ARGS_FILE)))
        content = load_toolchain_args(generators_folder=client.current_folder, namespace=namespace)
        for arg in check_args.get(namespace):
            checks.append(content.get(arg))
    client.run("build .")
    for check in checks:
        assert check in client.out



import textwrap

import pytest

from conans.test.assets.genconanfile import GenConanfile
from conans.test.integration.graph.core.graph_manager_base import GraphManagerTest
from conans.test.utils.tools import TestClient


class TestAlias(GraphManagerTest):

    def test_basic(self):
        # app -> liba/latest -(alias)-> liba/0.1
        self.recipe_cache("liba/0.1")
        self.alias_cache("liba/latest", "liba/0.1")

        consumer = self.recipe_consumer("app/0.1", ["liba/(latest)"])
        deps_graph = self.build_consumer(consumer)

        self.assertEqual(2, len(deps_graph.nodes))
        app = deps_graph.root
        liba = app.dependencies[0].dst

        self._check_node(liba, "liba/0.1#123", dependents=[app])
        self._check_node(app, "app/0.1", deps=[liba], closure=[liba])

    def test_alias_diamond(self):
        # app -> ----------------------------------> liba/0.1
        #   \ -> libb/0.1 -> liba/latest -(alias) ----->/
        self.recipe_cache("liba/0.1")
        self.alias_cache("liba/latest", "liba/0.1")
        self.recipe_cache("libb/0.1", requires=["liba/(latest)"])
        consumer = self.recipe_consumer("app/0.1", ["liba/0.1", "libb/0.1"])
        deps_graph = self.build_consumer(consumer)

        self.assertEqual(3, len(deps_graph.nodes))
        app = deps_graph.root
        liba = app.dependencies[0].dst
        libb = app.dependencies[1].dst

        self._check_node(liba, "liba/0.1#123", dependents=[app, libb])
        self._check_node(libb, "libb/0.1#123", deps=[liba], dependents=[app], closure=[liba])
        self._check_node(app, "app/0.1", deps=[liba, libb], closure=[libb, liba])

    def test_two_alias_diamond(self):
        # https://github.com/conan-io/conan/issues/3353
        # app -> liba/latest -(alias) --------------------------------------> liba/0.1
        #   \ -> libb/latest -(alias) -> libb/0.1 -> liba/latest -(alias) ----->/

        self.recipe_cache("liba/0.1")
        self.alias_cache("liba/latest", "liba/0.1")
        self.recipe_cache("libb/0.1", requires=["liba/(latest)"])
        self.alias_cache("libb/latest", "libb/0.1")

        consumer = self.recipe_consumer("app/0.1", ["liba/(latest)", "libb/(latest)"])
        deps_graph = self.build_consumer(consumer)

        self.assertEqual(3, len(deps_graph.nodes))
        app = deps_graph.root
        liba = app.dependencies[0].dst
        libb = app.dependencies[1].dst

        self._check_node(liba, "liba/0.1#123", dependents=[app, libb])
        self._check_node(libb, "libb/0.1#123", deps=[liba], dependents=[app], closure=[liba])
        self._check_node(app, "app/0.1", deps=[liba, libb], closure=[libb, liba])

    def test_full_two_branches_diamond(self):
        # https://github.com/conan-io/conan/issues/3353
        # app -> libb/latest -(alias) -> libb/0.1 -> liba/latest -(alias) ----> liba/0.1
        #   \ -> libc/latest -(alias) -> libc/0.1 -> liba/latest -(alias) ----->/

        self.recipe_cache("liba/0.1")
        self.alias_cache("liba/latest", "liba/0.1")
        self.recipe_cache("libb/0.1", requires=["liba/(latest)"])
        self.recipe_cache("libc/0.1", requires=["liba/(latest)"])
        self.alias_cache("libb/latest", "libb/0.1")
        self.alias_cache("libc/latest", "libc/0.1")

        consumer = self.recipe_consumer("app/0.1", ["libb/(latest)", "libc/(latest)"])
        deps_graph = self.build_consumer(consumer)

        self.assertEqual(4, len(deps_graph.nodes))
        app = deps_graph.root
        libb = app.dependencies[0].dst
        libc = app.dependencies[1].dst
        liba = libb.dependencies[0].dst

        self._check_node(liba, "liba/0.1#123", dependents=[libb, libc])
        self._check_node(libb, "libb/0.1#123", deps=[liba], dependents=[app], closure=[liba])
        self._check_node(libc, "libc/0.1#123", deps=[liba], dependents=[app], closure=[liba])
        self._check_node(app, "app/0.1", deps=[libb, libc], closure=[libb, libc, liba])

    def test_alias_bug(self):
        # https://github.com/conan-io/conan/issues/2252
        # app -> libb/0.1 -> liba/latest -(alias)->liba/0.1
        #   \ -> libc/0.1 ----/
        self.recipe_cache("liba/0.1")
        self.alias_cache("liba/latest", "liba/0.1")
        self.recipe_cache("libb/0.1", requires=["liba/(latest)"])
        self.recipe_cache("libc/0.1", requires=["liba/(latest)"])

        consumer = self.recipe_consumer("app/0.1", ["libb/0.1", "libc/0.1"])
        deps_graph = self.build_consumer(consumer)

        self.assertEqual(4, len(deps_graph.nodes))
        app = deps_graph.root
        libb = app.dependencies[0].dst
        libc = app.dependencies[1].dst
        liba = libb.dependencies[0].dst

        self._check_node(liba, "liba/0.1#123", dependents=[libb, libc])
        self._check_node(libb, "libb/0.1#123", deps=[liba], dependents=[app], closure=[liba])
        self._check_node(libc, "libc/0.1#123", deps=[liba], dependents=[app], closure=[liba])
        self._check_node(app, "app/0.1", deps=[libb, libc], closure=[libb, libc, liba])

    def test_alias_tansitive(self):
        # app -> liba/giga -(alias)->-> liba/mega -(alias)-> liba/latest -(alias)->liba/0.1

        self.recipe_cache("liba/0.1")
        self.alias_cache("liba/latest", "liba/0.1")
        self.alias_cache("liba/mega", "liba/(latest)")
        self.alias_cache("liba/giga", "liba/(mega)")

        consumer = self.recipe_consumer("app/0.1", ["liba/(giga)"])
        deps_graph = self.build_consumer(consumer)

        self.assertEqual(2, len(deps_graph.nodes))
        app = deps_graph.root
        liba = app.dependencies[0].dst

        self._check_node(liba, "liba/0.1#123", dependents=[app])
        self._check_node(app, "app/0.1", deps=[liba], closure=[liba])


class AliasBuildRequiresTest(GraphManagerTest):

    @pytest.mark.xfail(reason="This onely works with the build context")
    def test_non_conflicting_alias(self):
        # https://github.com/conan-io/conan/issues/5468
        # libc ----> libb -------------------> liba/0.1
        #   \-(build)-> liba/latest -(alias)-> liba/0.2
        self.recipe_cache("liba/0.1")
        self.recipe_cache("liba/0.2")
        self.alias_cache("liba/latest", "liba/0.2")
        self.recipe_cache("libb/0.1", ["liba/0.1"])
        consumer = self.recipe_consumer("app/0.1", ["libb/0.1"], build_requires=["liba/(latest)"])

        deps_graph = self.build_consumer(consumer)

        self.assertEqual(4, len(deps_graph.nodes))
        app = deps_graph.root
        libb = app.dependencies[0].dst
        liba_build = app.dependencies[1].dst
        liba = libb.dependencies[0].dst

        self._check_node(liba, "liba/0.1#123", dependents=[libb])
        self._check_node(liba_build, "liba/0.2#123", dependents=[app])
        self._check_node(libb, "libb/0.1#123", deps=[liba], dependents=[app])
        self._check_node(app, "app/0.1", deps=[libb, liba_build])


class AliasPythonRequiresTest(GraphManagerTest):

    def test_python_requires(self):
        # app ----(python-requires)---> tool/(latest) -> tool/0.1
        self.recipe_cache("tool/0.1")
        self.recipe_cache("tool/0.2")
        self.alias_cache("tool/latest", "tool/0.1")
        consumer = textwrap.dedent("""
            from conans import ConanFile
            class Pkg(ConanFile):
                name = "app"
                version = "0.1"
                python_requires = "tool/(latest)"
            """)
        deps_graph = self.build_graph(consumer)

        self.assertEqual(1, len(deps_graph.nodes))
        app = deps_graph.root
        self._check_node(app, "app/0.1", deps=[])


def test_mixing_aliases_and_fix_versions():
    # cd/1.0 -----------------------------> cb/latest -(alias)-> cb/1.0 -> ca/1.0
    #   \-----> cc/latest -(alias)-> cc/1.0 ->/                             /
    #                                    \------ ca/latest -(alias)------->/
    client = TestClient()

    client.save({"conanfile.py": GenConanfile("ca", "1.0")})
    client.run("create . ")
    client.run("alias ca/latest@ ca/1.0@")

    client.save({"conanfile.py": GenConanfile("cb", "1.0")
                .with_requirement("ca/1.0@")})
    client.run("create . cb/1.0@")
    client.run("alias cb/latest@ cb/1.0@")

    client.save({"conanfile.py": GenConanfile("cc", "1.0")
                .with_requirement("cb/(latest)")
                .with_requirement("ca/(latest)")})
    client.run("create . ")
    client.run("alias cc/latest@ cc/1.0@")

    client.save({"conanfile.py": GenConanfile("cd", "1.0")
                .with_requirement("cb/(latest)")
                .with_requirement("cc/(latest)")})
    client.run("create . ")



import platform
import textwrap

import pytest

from conans.test.utils.tools import TestClient


@pytest.fixture
def client():
    client = TestClient()
    conanfile = textwrap.dedent("""
        from conans import ConanFile
        from conan.tools.cmake import CMake

        class Pkg(ConanFile):
            settings = "os", "arch", "compiler", "build_type"
            generators = "CMakeToolchain"

            def run(self, cmd):  # INTERCEPTOR of running
                self.output.info("RECIPE-RUN: {}".format(cmd))

            def build(self):
                cmake = CMake(self)
                cmake.build()
        """)
    client.save({"conanfile.py": conanfile})
    return client


def test_cmake_no_config(client):
    profile = textwrap.dedent("""\
        [settings]
        os=Windows
        arch=x86_64
        compiler=Visual Studio
        compiler.version=15
        compiler.runtime=MD
        build_type=Release
        """)
    client.save({"myprofile": profile})
    client.run("create . pkg/0.1@ -pr=myprofile")
    assert "/verbosity" not in client.out


def test_cmake_config(client):
    profile = textwrap.dedent("""\
        [settings]
        os=Windows
        arch=x86_64
        compiler=Visual Studio
        compiler.version=15
        compiler.runtime=MD
        build_type=Release
        [conf]
        tools.microsoft.msbuild:verbosity=Minimal
        """)
    client.save({"myprofile": profile})
    client.run("create . pkg/0.1@ -pr=myprofile")
    assert "/verbosity:Minimal" in client.out


def test_cmake_config_error(client):
    profile = textwrap.dedent("""\
        [settings]
        os=Windows
        arch=x86_64
        compiler=Visual Studio
        compiler.version=15
        compiler.runtime=MD
        build_type=Release
        [conf]
        tools.microsoft.msbuild:verbosity=non-existing
        """)
    client.save({"myprofile": profile})
    client.run("create . pkg/0.1@ -pr=myprofile", assert_error=True)
    assert "Unknown msbuild verbosity: non-existing" in client.out


def test_cmake_config_package(client):
    profile = textwrap.dedent("""\
        [settings]
        os=Windows
        arch=x86_64
        compiler=Visual Studio
        compiler.version=15
        compiler.runtime=MD
        build_type=Release
        [conf]
        dep*:tools.microsoft.msbuild:verbosity=Minimal
        """)
    client.save({"myprofile": profile})
    client.run("create . pkg/0.1@ -pr=myprofile")
    assert "/verbosity" not in client.out
    client.run("create . dep/0.1@ -pr=myprofile")
    assert "/verbosity:Minimal" in client.out


def test_config_profile_forbidden(client):
    profile = textwrap.dedent("""\
        [conf]
        cache:verbosity=Minimal
        """)
    client.save({"myprofile": profile})
    client.run("install . pkg/0.1@ -pr=myprofile", assert_error=True)
    assert ("ERROR: Error reading 'myprofile' profile: [conf] "
            "'cache:verbosity' not allowed in profiles" in client.out)


def test_msbuild_config():
    client = TestClient()
    conanfile = textwrap.dedent("""
        from conans import ConanFile
        from conan.tools.microsoft import MSBuild

        class Pkg(ConanFile):
            settings = "os", "arch", "compiler", "build_type"
            def build(self):
                ms = MSBuild(self)
                self.output.info(ms.command("Project.sln"))
        """)
    client.save({"conanfile.py": conanfile})
    profile = textwrap.dedent("""\
        [settings]
        os=Windows
        arch=x86_64
        compiler=Visual Studio
        compiler.version=15
        compiler.runtime=MD
        build_type=Release
        [conf]
        tools.microsoft.msbuild:verbosity=Minimal
        """)
    client.save({"myprofile": profile})
    client.run("create . pkg/0.1@ -pr=myprofile")
    assert "/verbosity:Minimal" in client.out


@pytest.mark.tool_visual_studio
@pytest.mark.skipif(platform.system() != "Windows", reason="Only for windows")
def test_msbuild_compile_options():
    client = TestClient()
    conanfile = textwrap.dedent("""
        from conans import ConanFile

        class Pkg(ConanFile):
            settings = "os", "arch", "compiler", "build_type"
            generators = "MSBuildToolchain"
        """)
    client.save({"conanfile.py": conanfile})

    profile = textwrap.dedent("""\
        [settings]
        os=Windows
        arch=x86_64
        compiler=Visual Studio
        compiler.version=15
        compiler.runtime=MD
        build_type=Release
        [conf]
        tools.microsoft.msbuildtoolchain:compile_options={"ExceptionHandling": "Async"}
        """)
    client.save({"myprofile": profile})
    client.run("install . -pr=myprofile")
    msbuild_tool = client.load("conantoolchain_release_x64.props")
    assert "<ExceptionHandling>Async</ExceptionHandling>" in msbuild_tool
